@@ -47,6 +47,7 @@ struct fax {
 	unsigned int		more_doc:1;
 	unsigned int		no_ecm:1;
 	unsigned int		b3data_mapped:1;
+	unsigned int		B3DisconnectDone:1;
 	int			b3transfer_error;
 	int			b3_format;
 	unsigned char		*b3data;
@@ -77,7 +78,6 @@ static void StopDownLink(struct fax *fax)
 	struct mc_buf *mc;
 
 	dprint(MIDEBUG_NCCI, "NCCI %06x: StopDownLink\n", ncci->ncci);
-	ncciL4L3(ncci, PH_DEACTIVATE_REQ, 0, 0, NULL, NULL);
 	pthread_mutex_lock(&ncci->lock);
 	fax->modem_active = 0;
 	for (i = 0; i < ncci->window; i++) {
@@ -91,6 +91,10 @@ static void StopDownLink(struct fax *fax)
 	ncci->dlbusy = 0;
 	ncci->oidx = 0;
 	ncci->iidx = 0;
+	if (ncci->BIlink) {
+		ncci->BIlink->release_pending = 1;
+		ncciL4L3(ncci, PH_DEACTIVATE_REQ, 0, 0, NULL, NULL);
+	}
 	pthread_mutex_unlock(&ncci->lock);
 }
 
@@ -99,8 +103,11 @@ static void FaxB3Disconnect(struct fax *fax)
 	unsigned char ncpi[32], len, ncpi_len;
 	struct mc_buf *mc;
 
+	if (!fax || fax->B3DisconnectDone)
+		return;
 	mc = alloc_mc_buf();
 	if (mc) {
+		fax->B3DisconnectDone = 1;
 		ncciCmsgHeader(fax->ncci, mc, CAPI_DISCONNECT_B3, CAPI_IND);
 		ncpi_len = 8;
 		capimsg_setu16(ncpi, 1, fax->t30stats.bit_rate & 0xffff);
@@ -436,7 +443,6 @@ static void phaseE_handler(t30_state_t *t30, void *user_data, int result)
 		}
 	} else {
 		fax->b3transfer_error = 1;
-		FaxB3Disconnect(fax);
 	}
 }
 
@@ -611,6 +617,8 @@ static struct fax *mFaxCreate(struct BInstance	*bi)
 					wprint("Cannot InitFax for incoming PLCI %04x\n", bi->lp ? bi->lp->plci : 0xffff);
 					free(nf);
 					nf = NULL;
+					pthread_mutex_unlock(&bi->lp->lock);
+					return nf;
 				}
 			}
 			ncciL4L3(nf->ncci, PH_CONTROL_REQ, HW_FIFO_STATUS_ON, 0, NULL, NULL);
@@ -628,6 +636,7 @@ static struct fax *mFaxCreate(struct BInstance	*bi)
 				dprint(MIDEBUG_NCCI, "NCCI %06x: MISDN_CTRL_RX_BUFFER  values: min=%d -> %d max=%d\n",
 					nf->ncci->ncci, creq.p1, DEFAULT_PKT_SIZE, creq.p2);
 			nf->startdownlink = 1;
+			nf->modem_active = 1;
 		} else {
 			eprint("Cannot create NCCI for PLCI %04x\n", bi->lp ? bi->lp->plci : 0xffff);
 			free(nf);
@@ -765,6 +774,11 @@ static void FaxSendDown(struct fax *fax, int ind)
 	for (i = 0; i < ret; i++)
 		*p++ = slin2alaw[*w++];
 	pthread_mutex_lock(&ncci->lock);
+	if (!ncci->BIlink) {
+		wprint("NCCI %06x: BInstance is gone - packet ignored\n", ncci->ncci);
+		pthread_mutex_unlock(&ncci->lock);
+		return;
+	}
 	ncci->down_iv[1].iov_len = l;
 	ncci->down_iv[1].iov_base = sbuf;
 #endif
@@ -819,12 +833,17 @@ static int FaxDataInd(struct fax *fax, struct mc_buf *mc)
 	dlen = mc->len - sizeof(*hh);
 	pthread_mutex_lock(&ncci->lock);
 	if ((!ncci->BIlink) || (!fax->fax)) {
-		pthread_mutex_unlock(&ncci->lock);
-		wprint("NCCI %06x: frame with %d bytes dropped Blink(%p) or sandsp fax(%p) gone\n",
+		wprint("NCCI %06x: frame with %d bytes dropped Blink(%p) or spandsp fax(%p) gone\n",
 			ncci->ncci, dlen, ncci->BIlink, fax->fax);
+		pthread_mutex_unlock(&ncci->lock);
 		return -EINVAL;
 	} else
 		dprint(MIDEBUG_NCCI_DATA, "NCCI %06x: fax(%p)\n", ncci->ncci, fax->fax);
+	if (!fax->modem_active) {
+		wprint("NCCI %06x: Modem not active %d bytes dropped\n", ncci->ncci, dlen);
+		pthread_mutex_unlock(&ncci->lock);
+		return -ENOTCONN;
+	}
 	fst = fax->fax;
 	pthread_mutex_unlock(&ncci->lock);
 
@@ -892,14 +911,30 @@ static void FaxDataConf(struct fax *fax, struct mc_buf *mc)
 	struct mNCCI *ncci = fax->ncci;
 	struct mISDNhead *hh;
 
+	if (!fax || !ncci)
+		return;
+	pthread_mutex_lock(&ncci->lock);
+	if ((!ncci->BIlink) || (!fax->fax)) {
+		wprint("NCCI %06x: Blink(%p) or spandsp fax(%p) gone\n",
+			ncci->ncci, ncci->BIlink, fax->fax);
+		pthread_mutex_unlock(&ncci->lock);
+		free_mc_buf(mc);
+		return;
+	}
+	if (!fax->modem_active) {
+		wprint("NCCI %06x: Modem not active\n", ncci->ncci);
+		pthread_mutex_unlock(&ncci->lock);
+		free_mc_buf(mc);
+		return;
+	}
 	hh = (struct mISDNhead *)mc->rb;
 	if (ncci->flowmode != flmPHDATA) {
 		wprint("Fax NCCI %06x: Got DATA confirm for %x - but flow mode(%d)\n", ncci->ncci, hh->id, ncci->flowmode);
+		pthread_mutex_unlock(&ncci->lock);
 		free_mc_buf(mc);
 		return;
 	}
 
-	pthread_mutex_lock(&ncci->lock);
 	for (i = 0; i < ncci->window; i++) {
 		if (ncci->xmit_handles[i].PktId == hh->id) {
 			if (ncci->xmit_handles[i].pkt)
@@ -1272,6 +1307,15 @@ int FaxRecvBData(struct BInstance *bi, struct mc_buf *mc)
 	struct fax *fax = bi->b3data;
 	int ret = 0;
 
+	if (!mc) {
+		/* timeout RELEASE */
+		if (fax) {
+			fax->modem_end = 1;
+			fax->modem_active = 0;
+			FaxB3Disconnect(fax);
+		}
+		return 0;
+	}
 	hh = (struct mISDNhead *)mc->rb;
 	switch (hh->prim) {
 	case PH_DATA_IND:
@@ -1313,6 +1357,7 @@ int FaxRecvBData(struct BInstance *bi, struct mc_buf *mc)
 		}
 		fax->modem_end = 1;
 		fax->modem_active = 0;
+		FaxB3Disconnect(fax);
 		ret = 1;
 		break;
 	case PH_DEACTIVATE_CNF:
@@ -1324,6 +1369,7 @@ int FaxRecvBData(struct BInstance *bi, struct mc_buf *mc)
 		}
 		fax->modem_end = 1;
 		fax->modem_active = 0;
+		FaxB3Disconnect(fax);
 		ret = 1;
 		break;
 	case PH_CONTROL_CNF:
