@@ -271,9 +271,10 @@ static void lPLCIInfoIndIE(struct lPLCI *lp, unsigned char ie, uint32_t mask, st
 	Send2Application(lp, mc);
 }
 
-uint16_t q931CIPValue(struct mc_buf * mc)
+uint16_t q931CIPValue(struct mc_buf * mc, uint32_t *m)
 {
 	uint16_t CIPValue = 0;
+	uint32_t CIPMask;
 	int capability, mode, rate, oct5;
 	int hlc = -1, ehlc = -1;
 	int ret, l;
@@ -338,6 +339,8 @@ uint16_t q931CIPValue(struct mc_buf * mc)
 		return 0;
 	}
 
+	CIPMask = 1 << CIPValue;
+
 	if (mc->l3m->hlc) {
 		ret = mi_decode_hlc(mc->l3m, &hlc, &ehlc);
 		l = *mc->l3m->hlc;
@@ -402,6 +405,9 @@ uint16_t q931CIPValue(struct mc_buf * mc)
 		} else
 			wprint("Cannot decode HLC %s return %d - %s\n", bdebug, ret, strerror(-ret));
 	}
+	CIPMask |= 1 << CIPValue;
+	if (m)
+		*m = CIPMask;
 	return CIPValue;
 }
 
@@ -705,7 +711,7 @@ static void plci_connect_resp(struct FsmInst *fi, int event, void *arg)
 	struct mPLCI *plci = lp->PLCI;
 	struct mc_buf *mc = arg;
 	struct l3_msg *l3m;
-	int cause;
+	int cause = 0;
 
 	if (mc->cmsg.Reject == 0) {	// accept
 		if (lPLCICheckBprotocol(lp, &mc->cmsg)) {
@@ -721,33 +727,34 @@ static void plci_connect_resp(struct FsmInst *fi, int event, void *arg)
 	// ignore, reject 
 	switch (mc->cmsg.Reject) {
 	case 1:
-		break;		// Ignore - no message should be sent
+		cause = CAUSE_NOUSER_RESPONDING;
+		break;		// Ignore
 	case 2:
-		cause = 0x90;
-		break;		// normal call clearing
+		cause = CAUSE_NORMAL_CLEARING;
+		break;
 	case 3:
-		cause = 0x91;
-		break;		// user busy
+		cause = CAUSE_USER_BUSY;
+		break;
 	case 4:
-		cause = 0xac;
-		break;		// req circuit/channel not avail
+		cause = CAUSE_REQUESTED_CHANNEL;
+		break;
 	case 5:
-		cause = 0x9d;
-		break;		// fac rejected
+		cause = CAUSE_FACILITY_REJECTED;
+		break;
 	case 6:
-		cause = 0x86;
-		break;		// channel unacceptable
+		cause = CAUSE_CHANNEL_UNACCEPT;
+		break;
 	case 7:
-		cause = 0xd8;
-		break;		// incompatible dest
+		cause = CAUSE_INCOMPATIBLE_DEST;
+		break;
 	case 8:
-		cause = 0x9b;
-		break;		// dest out of order
+		cause = CAUSE_DEST_OUT_OF_ORDER;
+		break;
 	default:
 		if ((mc->cmsg.Reject & 0xff00) == 0x3400) {
-			cause = mc->cmsg.Reject & 0xff;
+			cause = mc->cmsg.Reject & 0x7f;
 		} else {
-			cause = 0x90;	// normal call clearing
+			cause = CAUSE_NORMAL_CLEARING;	// normal call clearing
 		}
 	}
 	// FIXME
@@ -757,7 +764,7 @@ static void plci_connect_resp(struct FsmInst *fi, int event, void *arg)
 	//      lPLCIClearOtherApps(lp);
 	// }
 	// plciDetachlPLCI(plci, lp);
-	if (mc->cmsg.Reject != 1) {
+	if (mc->cmsg.Reject != 0) {
 		if (plci->nAppl == 1) {
 			int mt;
 
@@ -967,7 +974,7 @@ static void plci_cc_setup_ind(struct FsmInst *fi, int event, void *arg)
 
 	// FIXME: CW
 	if (mc->l3m) {
-		mc->cmsg.CIPValue = q931CIPValue(mc);
+		mc->cmsg.CIPValue = q931CIPValue(mc, NULL);
 		if (mc->l3m->called_nr && *mc->l3m->called_nr)
 			mc->cmsg.CalledPartyNumber = mc->l3m->called_nr;
 		if (mc->l3m->called_sub && *mc->l3m->called_sub)
