@@ -213,7 +213,10 @@ static void ncci_disconnect_b3_req(struct FsmInst *fi, int event, void *arg)
 		FsmChangeState(fi, ST_NCCI_N_4);
 	}
 	prim = ncci->l1direct ? PH_DEACTIVATE_REQ : DL_RELEASE_REQ;
-	ncciL4L3(ncci, prim, 0, 0, NULL, NULL);
+	if (ncci->BIlink) {
+		ncci->BIlink->release_pending = 1;
+		ncciL4L3(ncci, prim, 0, 0, NULL, NULL);
+	}
 }
 
 static void ncci_disconnect_b3_conf(struct FsmInst *fi, int event, void *arg)
@@ -421,13 +424,41 @@ static void ncci_appl_release(struct FsmInst *fi, int event, void *arg)
 	ncciFree(fi->userdata);
 }
 
+static void ncci_clearing_stateN_0(struct FsmInst *fi, int event, void *arg)
+{
+	struct mNCCI *ncci = fi->userdata;
+	struct mc_buf *mc = alloc_mc_buf();
+	int ret;
+
+	FsmChangeState(fi, ST_NCCI_N_5);
+	if (mc) {
+		if (ncci->BIlink) {
+			ncciCmsgHeader(ncci, mc, CAPI_DISCONNECT_B3, CAPI_RESP);
+			capi_cmsg2message(&mc->cmsg, mc->rb);
+			mc->len = CAPIMSG_LEN(mc->rb);
+			ret = ncci->BIlink->from_up(ncci->BIlink, mc);
+		} else
+			ret = -EINVAL;
+		if (ret) { /* no handler - free anyway */
+			ncciFree(ncci);
+			free_mc_buf(mc);
+		}
+	} else {
+		eprint("Cannot allocate buffer\n");
+		ncciFree(ncci);
+	}
+}
+
 static void ncci_appl_release_disc(struct FsmInst *fi, int event, void *arg)
 {
 	struct mNCCI *ncci = fi->userdata;
 	int prim;
 
 	prim = ncci->l1direct ? PH_DEACTIVATE_REQ : DL_RELEASE_REQ;
-	ncciL4L3(ncci, prim, 0, 0, NULL, NULL);
+	if (ncci->BIlink) {
+		ncci->BIlink->release_pending = 1;
+		ncciL4L3(ncci, prim, 0, 0, NULL, NULL);
+	}
 }
 
 static struct FsmNode fn_ncci_list[] = {
@@ -436,6 +467,9 @@ static struct FsmNode fn_ncci_list[] = {
 	{ST_NCCI_N_0, EV_DL_ESTABLISH_CONF, ncci_n0_dl_establish_ind_conf},
 	{ST_NCCI_N_0, EV_DL_ESTABLISH_IND, ncci_n0_dl_establish_ind_conf},
 	{ST_NCCI_N_0, EV_AP_RELEASE, ncci_appl_release},
+	{ST_NCCI_N_0, EV_NC_DISCONNECT_B3_IND, ncci_clearing_stateN_0},
+	{ST_NCCI_N_0, EV_DL_RELEASE_IND, ncci_clearing_stateN_0},
+	{ST_NCCI_N_0, EV_NC_LINKDOWN, ncci_clearing_stateN_0},
 
 	{ST_NCCI_N_0_1, EV_NC_CONNECT_B3_CONF, ncci_connect_b3_conf},
 	{ST_NCCI_N_0_1, EV_AP_MANUFACTURER_REQ, ncci_manufacturer_req},
@@ -1226,6 +1260,8 @@ int ncciL4L3(struct mNCCI *ncci, uint32_t prim, int id, int len, void *data, str
 	struct mISDNhead *hh;
 	int ret, l = sizeof(*hh);
 
+	if (!ncci->BIlink)
+		return -ENOTCONN;
 	dprint(MIDEBUG_NCCI, "NCCI %06x: prim %s id %x len %d\n", ncci->ncci, _mi_msg_type2str(prim), id, len);
 	if (!mc) {
 		loc = alloc_mc_buf();
@@ -1263,6 +1299,11 @@ int recvBdirect(struct BInstance *bi, struct mc_buf *mc)
 	struct mNCCI *ncci = bi->b3data;
 	int ret = 0;
 
+	if (!mc) {
+		/* timeout release */
+		ncciReleaseLink(ncci);
+		return 0;
+	}
 	hh = (struct mISDNhead *)mc->rb;
 	switch (hh->prim) {
 	// we're not using the Fsm and _cmesg coding for DL_DATA for performance reasons
